function [xProj, projType] = project_L1LInf(x, tau1, tau2, groups, w)

% We assume x>=0.
verbose = 0;
nRow = max(groups);
nCol = length(groups) / nRow;
X = reshape(x,nRow,nCol);
if nargin < 6
    w = ones(nRow,1);
end

norm_l1_l1inf = sum(max(X,[],2));
norm_l1 = sum(x);

%% Trivial Cases
if norm_l1_l1inf <= tau1 && norm_l1 <= tau2
    % Need not to project
    XProj = X;
    xProj = XProj(:);
    projType = 0;
    if verbose
        fprintf('Need not to project!\n');
    end
    return;
elseif norm_l1_l1inf > tau1 && norm_l1 <= tau2
    % Just need L1Inf projection
    XProj = MyL1InfProjection(X, tau1, w);
    xProj = XProj(:);
    projType = 1;
    if verbose
        fprintf('apply L1Inf projection!\n');
    end
    return;
elseif norm_l1_l1inf <= tau1 && norm_l1 > tau2
    % Just need L1 projection
    xProj = projectRandom2C(x, tau2);
    XProj = reshape(xProj, nRow, nCol);
    projType = 2;
    if verbose
        fprintf('apply L1 projection!\n');
    end
    return;
else
    % Just apply L1 projection and check whether it satisfies
    xProj = projectRandom2C(x, tau2);
    XProj = reshape(xProj, nRow, nCol);
    norm_l1_l1inf = sum(max(XProj,[],2));   
    if norm_l1_l1inf <= tau1
        % L1 projection satisfies
        projType = 2;
        if verbose
            fprintf('apply L1 projection!\n');
        end
        return;      
    else          
        % Just apply L1Inf projection and check whether it satisfies
        XProj = MyL1InfProjection(X, tau1, w);
        xProj = XProj(:);
        norm_l1 = sum(sum(XProj));
        if norm_l1 <= tau2
            % L1Inf projection satisfies
            projType = 1;
            if verbose
                fprintf('apply LInf projection!\n');
            end
            return;           
        else
            % apply L1_L1Inf projection
            XProj = project_L1LInf_core(X, tau1, tau2, w);
            xProj = XProj(:);
            norm_l1_l1inf = sum(max(XProj,[],2));
            norm_l1 = sum(sum(XProj));
            if norm_l1_l1inf > tau1 + 1e-6 || norm_l1 > tau2 + 1e-6
                fprintf('L1+LInf projection fails!\n');
                save('failure_case_l1linf_proj.mat');
                pause;
            end
            projType = 3;
            if verbose
                fprintf('apply L1+LInf projection!\n');
            end
        end
    end
end