function [ obj ] = compute_objective( X, Sigma, objtype, lambda )
% X: n x p design pool
% Sigma: X'*diag(w)*X, the re-weighted sample covariance
% objtype: {'A', 'D', 'T', 'E', 'I', 'G'}
% lambda: smoothing parameter. default 0.
% Note: for D-optimality, -1/p log det(Sigma) is reported
    
    if (nargin < 4)
        lambda = 0;
    end

    if (sum(sum(isnan(Sigma))) + sum(sum(isinf(Sigma))) > 0)
        obj = nan;
        return;
    end

    Sigma = Sigma + lambda * eye(size(Sigma,1));

    % if Sigma is close to singular, return inf
    if (cond(Sigma) > 1e10)
        obj = inf;
        return;
    end
    
    [n, p] = size(X);

    switch objtype
        case {'a', 'A'}
            obj = 1/p * trace(inv(Sigma));
        case {'d', 'D'}
            obj = -1/p * logdet(Sigma, 'chol');
        case {'t', 'T'}
            obj = p / trace(Sigma);
        case {'e', 'E'}
            obj = norm(inv(Sigma), 2);
            %obj = -1/norm(inv(Sigma),2);
        case {'i', 'I'}
            obj = 1/n * trace(X * (Sigma \ X'));
        case {'g', 'G'}
            obj = max(diag(X * (Sigma \ X')));
        otherwise
            error(['Unrecognized objtype: ', objtype]);
    end

end

