function [ C ] = fedorov( X, k, objtype, withrep, verbose, maxT )
% Implement the Fedorov's exchange algorithm
% Warning: this routine can be slow. Test on small input first.
% X: n x p design matrix
% k: target subset size
% objtype: {A, T, G, E, I, V}
% withrep: is duplicates allowed in the subsampled set?
% verbose: set to true for diagnostic information
% maxT: maximum # of exchanges allowed. default 100.
% return: C, a subset of k indices selected

    if (nargin < 6)
        maxT = 100;
    end

    if (nargin < 5)
        verbose = 1;
    end

    [n, p] = size(X);
	[Xs, C0] = datasample(X, k, 'Replace', false);
    A = Xs' * Xs;
	if (cond(A) < 1e10)
	    logdetA = logdet(A, 'chol');
	else
		C = C0;
		return;
	end
    cov0 = 1/n * (X'*X);
    invA = inv(A);
    
%     f = fopen('input.txt', 'w');
%     printmat(f, X);
%     printmat(f, double(C0));
%     printmat(f, A);
%     printmat(f, invA);
%     fprintf(f, '%.12f\n', logdetA);
%     printmat(f, cov0);
%     fclose(f);
    
    C = fedorov_c(objtype, X', double(C0), A, invA, logdetA, cov0, withrep, verbose, maxT);

end

