function [ ret ] = regsel_heuristic( X, D, k, param_eps, param_delta, thresh )
% A simplified version of the multi-stage sparsification algorithm
% described in the ICML paper. Applicable to without replacement only.
% Input:
%   - thresh: a real value in [0,1]. Include all points whose weight pi_i^*
%   exceeds this threshold, and run regsel.m with the remaining budget (if
%   there is any).
%   - refer to regsel.m for explanations of the other parameters.
% Output:
%   - ret: a binary n x 1 vector that sums to k

    n = size(X, 1);
    ret = zeros(n, 1);
    [weights, inds] = sort(D, 'descend');
    i = 1;
    while (sum(ret) < k && weights(i) >= thresh)
        ret(inds(i)) = 1;
        D(inds(i)) = 0;
        i = i + 1;
    end
  
    if (sum(ret) < k)
        if (k - sum(ret) < k)
            % not enough budget left. The only thing to do is to continue
            % greedy selection
            while (sum(ret) < k)
                ret(inds(i)) = 1;
                i = i + 1;
            end
        else
            t_ret = regsel(X, D, k-sum(ret), false, param_eps, param_delta, false);
            ret = ret + t_ret;
        end
    end

end

