function [ gamma_star ] = select_psg_stepsize( X, infty_norm, objtype, lambda, T, verbose )

    if (nargin < 5)
        verbose = false;
    end
    if (nargin < 4)
        T = 1000;
    end
    
    gammas = [1e-2 3e-3 1e-3 3e-4 1e-4 3e-5 1e-5 3e-6 1e-6];
    
    TG = length(gammas);
    errs = zeros(1, TG);
    for tg = 1:TG
        [w, tot_iter, f_opt] = solve_pi_psg(X, gammas(tg), infty_norm, objtype, lambda, T, 100);
        if (verbose)
            fprintf('Gamma = %f: f_opt(%d) = %f\n', gammas(tg), T, f_opt);
        end
        errs(tg) = f_opt;
    end
    
    [min_err, id_err] = min(errs);
    gamma_star = gammas(id_err);
    fprintf('Stepsize selection done: gamma_star = %f, f_opt(%d) = %f\n', gamma_star, T, min_err);
    if (min_err == inf)
        gamma_star = nan;
    end

end

